/*-
 * Copyright (c) 2003-2007 Tim Kientzle
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "test.h"
__FBSDID("$FreeBSD: src/lib/libarchive/test/test_compat_gtar.c,v 1.4 2008/12/17 19:05:00 kientzle Exp $");

/*
 * Verify our ability to read sample files created by GNU tar.
 * It should be easy to add any new sample files sent in by users
 * to this collection of tests.
 */

/* Copy this function for each test file and adjust it accordingly. */

/*
 * test_compat_gtar_1.tgz exercises reading long filenames and
 * symlink targets stored in the GNU tar format.
 */
static void
test_compat_gtar_1(void)
{
    char name[] = "test_compat_gtar_1.tar";
    struct archive_entry *ae;
    struct archive *a;
    int r;

    assert((a = archive_read_new()) != NULL);
    assertEqualIntA(a, ARCHIVE_OK, archive_read_support_compression_all(a));
    assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
    extract_reference_file(name);
    assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, name, 10240));

    /* Read first entry. */
    assertEqualIntA(a, ARCHIVE_OK, r = archive_read_next_header(a, &ae));
    if (r != ARCHIVE_OK) {
        archive_read_finish(a);
        return;
    }
    assertEqualString(
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890",
        archive_entry_pathname(ae));
    assertEqualInt(1197179003, archive_entry_mtime(ae));
    assertEqualInt(1000, archive_entry_uid(ae));
    assertEqualString("tim", archive_entry_uname(ae));
    assertEqualInt(1000, archive_entry_gid(ae));
    assertEqualString("tim", archive_entry_gname(ae));
    assertEqualInt(0100644, archive_entry_mode(ae));

    /* Read second entry. */
    assertEqualIntA(a, ARCHIVE_OK, r = archive_read_next_header(a, &ae));
    if (r != ARCHIVE_OK) {
        archive_read_finish(a);
        return;
    }
    assertEqualString(
        "abcdefghijabcdefghijabcdefghijabcdefghijabcdefghij"
        "abcdefghijabcdefghijabcdefghijabcdefghijabcdefghij"
        "abcdefghijabcdefghijabcdefghijabcdefghijabcdefghij"
        "abcdefghijabcdefghijabcdefghijabcdefghijabcdefghij",
        archive_entry_pathname(ae));
    assertEqualString(
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890"
        "12345678901234567890123456789012345678901234567890",
        archive_entry_symlink(ae));
    assertEqualInt(1197179043, archive_entry_mtime(ae));
    assertEqualInt(1000, archive_entry_uid(ae));
    assertEqualString("tim", archive_entry_uname(ae));
    assertEqualInt(1000, archive_entry_gid(ae));
    assertEqualString("tim", archive_entry_gname(ae));
    assertEqualInt(0120755, archive_entry_mode(ae));

    /* Verify the end-of-archive. */
    assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));

    /* Verify that the format detection worked. */
    assertEqualInt(archive_compression(a), ARCHIVE_COMPRESSION_NONE);
    assertEqualInt(archive_format(a), ARCHIVE_FORMAT_TAR_GNUTAR);

    assertEqualInt(ARCHIVE_OK, archive_read_close(a));
#if ARCHIVE_VERSION_NUMBER < 2000000
    archive_read_finish(a);
#else
    assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
#endif
}


DEFINE_TEST(test_compat_gtar)
{
    test_compat_gtar_1();
}


