/*-
 * Copyright (c) 2009 Michihiro NAKAJIMA
 * Copyright (c) 2003-2008 Tim Kientzle
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "test.h"
__FBSDID("$FreeBSD$");

/*
 * Verify our ability to read sample files compatibly with unxz.
 *
 * In particular:
 *  * unxz will read multiple xz streams, concatenating the output
 */

/*
 * All of the sample files have the same contents; they're just
 * compressed in different ways.
 */
static void
compat_xz(const char *name)
{
    const char *n[7] = { "f1", "f2", "f3", "d1/f1", "d1/f2", "d1/f3", NULL };
    struct archive_entry *ae;
    struct archive *a;
    int i, r;

    assert((a = archive_read_new()) != NULL);
    assertEqualIntA(a, ARCHIVE_OK, archive_read_support_compression_all(a));
    r = archive_read_support_compression_xz(a);
    if (r == ARCHIVE_WARN) {
        skipping("xz reading not fully supported on this platform");
        assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
        return;
    }
    assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
    extract_reference_file(name);
    assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, name, 2));

    /* Read entries, match up names with list above. */
    for (i = 0; i < 6; ++i) {
        failure("Could not read file %d (%s) from %s", i, n[i], name);
        assertEqualIntA(a, ARCHIVE_OK,
            archive_read_next_header(a, &ae));
        assertEqualString(n[i], archive_entry_pathname(ae));
    }

    /* Verify the end-of-archive. */
    assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));

    /* Verify that the format detection worked. */
    assertEqualInt(archive_compression(a), ARCHIVE_COMPRESSION_XZ);
    assertEqualString(archive_compression_name(a), "xz");
    assertEqualInt(archive_format(a), ARCHIVE_FORMAT_TAR_USTAR);

    assertEqualInt(ARCHIVE_OK, archive_read_close(a));
    assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
}


DEFINE_TEST(test_compat_xz)
{
    compat_xz("test_compat_xz_1.txz");
}
