/*-
 * Copyright (c) 2003-2007 Tim Kientzle
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "test.h"
__FBSDID("$FreeBSD: src/lib/libarchive/test/test_read_format_iso_gz.c,v 1.2 2008/09/01 05:38:33 kientzle Exp $");

DEFINE_TEST(test_read_format_iso_gz)
{
    struct archive_entry *ae;
    struct archive *a;
    int r;
    const char *name = "test_read_format_iso_gz.iso.gz";

    extract_reference_file(name);

    assert((a = archive_read_new()) != NULL);
    assertEqualIntA(a, ARCHIVE_OK,
        archive_read_support_compression_all(a));
    r = archive_read_support_compression_gzip(a);
    if (r == ARCHIVE_WARN) {
        skipping("gzip reading not fully supported on this platform");
        assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
        return;
    }
    assertEqualIntA(a, ARCHIVE_OK,
        archive_read_support_format_all(a));
    assertEqualIntA(a, ARCHIVE_OK,
        archive_read_open_filename(a, name, 512));
    assertEqualIntA(a, ARCHIVE_OK,
        archive_read_next_header(a, &ae));
    assertEqualInt(archive_compression(a),
        ARCHIVE_COMPRESSION_GZIP);
    assertEqualInt(archive_format(a), ARCHIVE_FORMAT_ISO9660);
    assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
    assertEqualInt(ARCHIVE_OK, archive_read_finish(a));
}


