/*-
 * Copyright (c) 2007 Tim Kientzle
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "test.h"
__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_compress.c,v 1.4 2008/12/17 19:05:25 kientzle Exp $");

/*
 * A basic exercise of compress reading and writing.
 *
 * TODO: Add a reference file and make sure we can decompress that.
 */

DEFINE_TEST(test_write_compress)
{
    struct archive_entry *ae;
    struct archive* a;
    char *buff, *data;
    size_t buffsize, datasize;
    char path[16];
    size_t used;
    int i;

    buffsize = 1000000;
    assert(NULL != (buff = (char *)malloc(buffsize)));

    datasize = 10000;
    assert(NULL != (data = (char *)malloc(datasize)));
    memset(data, 0, datasize);

    assert((a = archive_write_new()) != NULL);
    assertA(0 == archive_write_set_format_ustar(a));
    assertA(0 == archive_write_set_compression_compress(a));
    assertA(0 == archive_write_open_memory(a, buff, buffsize, &used));

    for (i = 0; i < 100; i++) {
        sprintf(path, "file%03d", i);
        assert((ae = archive_entry_new()) != NULL);
        archive_entry_copy_pathname(ae, path);
        archive_entry_set_size(ae, datasize);
        archive_entry_set_filetype(ae, AE_IFREG);
        assertA(0 == archive_write_header(a, ae));
        assertA(datasize == (size_t)archive_write_data(a, data, datasize));
        archive_entry_free(ae);
    }


    archive_write_close(a);
#if ARCHIVE_VERSION_NUMBER < 2000000
    archive_write_finish(a);
#else
    assert(0 == archive_write_finish(a));
#endif

    /*
     * Now, read the data back.
     */
    assert((a = archive_read_new()) != NULL);
    assertA(0 == archive_read_support_format_all(a));
    assertA(0 == archive_read_support_compression_all(a));
    assertA(0 == archive_read_open_memory(a, buff, used));


    for (i = 0; i < 100; i++) {
        sprintf(path, "file%03d", i);
        if (!assertEqualInt(0, archive_read_next_header(a, &ae)))
            break;
        assertEqualString(path, archive_entry_pathname(ae));
        assertEqualInt((int)datasize, archive_entry_size(ae));
    }
    assert(0 == archive_read_close(a));
#if ARCHIVE_VERSION_NUMBER < 2000000
    archive_read_finish(a);
#else
    assert(0 == archive_read_finish(a));
#endif

    free(data);
    free(buff);
}
