/*-
 * Copyright (c) 2003-2007 Tim Kientzle
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "test.h"
__FBSDID("$FreeBSD: src/lib/libarchive/test/test_write_format_tar_empty.c,v 1.4 2008/09/01 05:38:33 kientzle Exp $");

/*
 * Check that an "empty" tar archive is correctly created.
 */

DEFINE_TEST(test_write_format_tar_empty)
{
    struct archive *a;
    char buff[2048];
    size_t used;
    unsigned int i;

    /* USTAR format: Create a new archive in memory. */
    assert((a = archive_write_new()) != NULL);
    assertA(0 == archive_write_set_format_ustar(a));
    assertA(0 == archive_write_set_compression_none(a));
    assertA(0 == archive_write_set_bytes_per_block(a, 512));
    assertA(0 == archive_write_set_bytes_in_last_block(a, 512));
    assertA(0 == archive_write_open_memory(a, buff, sizeof(buff), &used));

    /* Close out the archive. */
    assertA(0 == archive_write_close(a));
#if ARCHIVE_VERSION_NUMBER < 2000000
    archive_write_finish(a);
#else
    assertA(0 == archive_write_finish(a));
#endif

#if ARCHIVE_VERSION_NUMBER < 1009000
    /* Earlier versions wrote 0-length files for empty tar archives. */
    skipping("empty tar archive size");
#else
    assert(used == 1024);
#endif
    for (i = 0; i < used; i++) {
        failure("Empty tar archive should be all nulls.");
        assert(buff[i] == 0);
    }

    /* PAX format: Create a new archive in memory. */
    assert((a = archive_write_new()) != NULL);
    assertA(0 == archive_write_set_format_pax(a));
    assertA(0 == archive_write_set_compression_none(a));
    assertA(0 == archive_write_set_bytes_per_block(a, 512));
    assertA(0 == archive_write_set_bytes_in_last_block(a, 512));
    assertA(0 == archive_write_open_memory(a, buff, sizeof(buff), &used));

    /* Close out the archive. */
    assertA(0 == archive_write_close(a));
#if ARCHIVE_VERSION_NUMBER < 2000000
    archive_write_finish(a);
#else
    assertA(0 == archive_write_finish(a));
#endif

#if ARCHIVE_VERSION_NUMBER < 1009000
    /* Earlier versions wrote 0-length files for empty tar archives. */
    skipping("empty tar archive size");
#else
    assertEqualInt((int)used, 1024);
#endif
    for (i = 0; i < used; i++) {
        failure("Empty tar archive should be all nulls.");
        assert(buff[i] == 0);
    }
}
